@@ -6,15 +6,78 @@ from ..kernel import CommandMatchRejected, Modifier
 # any changes, unless the user has already changed away from the default.
 # If you want to delete a bind / alias, set the first value to a null string.
 
+# CHOICE OF BINDALIAS vs. MENU
+# These key bindings are in addition to those specified in the menus
+# (like Ctrl+S for File Save). The choice of which of these to use is debatable.
+# On one hand bindalias is a more flexible approach because the user can
+# change keys mappings to whatever they wish whilst menus are fixed.
+# On the other hand, default menu keys can be set per language because the
+# menu strings are translated, whereas bindalias keys are not.
+
+# RESERVED KEYS
+# Regardless of the platform used by individual developers, bindalias needs
+# to conform to BOTH of the following:
+#
+# 1. Mac - common shortcuts defined in https://support.apple.com/en-us/HT201236
+#    should be followed where appropriate and avoided if not. This is important
+#    for MK to be accepted into the Apple app store.
+#    Where keys defined in other sections on this page make sense for this application,
+#    these should also be used by preference.
+#
+# 2. Windows - alt-single letter should be avoided as these correspond to menu
+#    shortcuts (e.g. in english locale, alt-f should activate the file menu).
+#    Common Windows shortcuts can be found at
+#    https://support.microsoft.com/en-us/windows/keyboard-shortcuts-in-windows-dcc61a57-8ff0-cffe-9796-cb9706c75eec
+#
+# In addition where they do not conflict with the above, any synergy with keys commonly used
+# in popular SVG / image editors (e.g. paint.net, inkscape) may be beneficial.
+
+# To change keymaps / alias insert new alias at the beginning of the tuple
+# Later entries in the tuple are used to identify previous defaults and update them to current,
+# so do not delete these until a version change (like 0.8) results in completely new settings anyway.
 DEFAULT_KEYMAP = {
-    "escape": ("pause",),
-    "pause": ("pause",),
-    "d": ("+right",),
     "a": ("+left",),
+    "d": ("+right",),
     "w": ("+up",),
     "s": ("+down",),
     "l": ("lock",),
     "u": ("unlock",),
+    "f4": (
+        "",
+        "window open CameraInterface",
+    ),
+    "f5": ("refresh",),
+    "f6": (
+        "",
+        "window open JobSpooler",
+    ),
+    "f7": (
+        "",
+        "window open -o Controller",
+        "window controller",
+        "window open Controller",
+    ),
+    "f8": ("", "dialog_path", "control Path"),
+    "f9": (
+        "",
+        "dialog_transform",
+        "control Transform",
+    ),
+    "f12": (
+        "",
+        "window open Console",
+        "window open Terminal",
+    ),
+    "delete": (
+        "tree emphasized delete",
+        "tree selected delete",
+        "element delete",
+    ),
+    "escape": (
+        "",
+        "pause",
+    ),
+    "home": ("home",),
     "numpad_down": ("+translate_down",),
     "numpad_up": ("+translate_up",),
     "numpad_left": ("+translate_left",),
@@ -23,51 +86,106 @@ DEFAULT_KEYMAP = {
     "numpad_divide": ("+scale_down",),
     "numpad_add": ("+rotate_cw",),
     "numpad_subtract": ("+rotate_ccw",),
-    "control+a": ("element* select",),
-    "control+c": ("clipboard copy",),
-    "control+v": ("clipboard paste",),
-    "control+x": ("clipboard cut",),
-    "control+i": ("element* select^",),
-    "control+f": ("", "dialog_fill", "control Fill",),
-    "control+s": ("", "dialog_stroke", "control Stroke",),
-    "alt+f": ("dialog_fill",),
-    "alt+p": ("dialog_flip",),
-    "alt+s": ("dialog_stroke",),
-    "alt+h": ("dialog_path",),
-    "alt+t": ("dialog_transform",),
-    "control+r": ("rect 0 0 1000 1000",),
-    "control+e": ("circle 500 500 500",),
-    "control+d": ("element copy",),
-    "control+o": ("outline",),
-    "control+shift+o": ("outline 1mm",),
-    "control+alt+o": ("outline -1mm",),
-    "control+shift+h": ("scale -1 1",),
-    "control+shift+v": ("scale 1 -1",),
-    "control+1": ("bind 1 move $x $y",),
-    "control+2": ("bind 2 move $x $y",),
-    "control+3": ("bind 3 move $x $y",),
-    "control+4": ("bind 4 move $x $y",),
-    "control+5": ("bind 5 move $x $y",),
-    "alt+r": ("raster",),
-    "alt+e": ("engrave",),
-    "alt+c": ("cut",),
-    "delete": ("tree selected delete", "element delete",),
-    "control+f3": ("", "rotaryview",),
-    "alt+f3": ("", "rotaryscale",),
-    "f4": ("", "window open CameraInterface",),
-    "f5": ("refresh",),
-    "f6": ("", "window open JobSpooler",),
-    "f7": ("", "window open -o Controller", "window controller", "window open Controller",),
-    "f8": ("", "dialog_path", "control Path"),
-    "f9": ("", "dialog_transform", "control Transform",),
-    "control+f9": ("", "dialog_flip", "control Flip",),
-    "f12": ("", "window open Console", "window open Terminal",),
-    "control+alt+g": ("image wizard Gold",),
-    "control+alt+x": ("image wizard Xin",),
-    "control+alt+s": ("image wizard Stipo",),
-    "home": ("home",),
-    "control+z": ("reset",),
-    "control+alt+shift+escape": ("reset_bind_alias",),
+    "pause": ("pause",),
+    "alt+c": (
+        "",
+        "cut",
+    ),
+    "alt+e": (
+        "",
+        "engrave",
+    ),
+    "alt+f": (
+        "",
+        "dialog_fill",
+    ),
+    "alt+h": (
+        "",
+        "dialog_path",
+    ),
+    "alt+p": (
+        "",
+        "dialog_flip",
+    ),
+    "alt+r": (
+        "",
+        "raster",
+    ),
+    "alt+s": (
+        "",
+        "dialog_stroke",
+    ),
+    "alt+t": (
+        "",
+        "dialog_transform",
+    ),
+    "alt+f3": (
+        "",
+        "rotaryscale",
+    ),
+    "ctrl+a": ("element* select",),
+    "ctrl+c": ("clipboard copy",),
+    "ctrl+e": ("circle 500 500 500",),
+    "ctrl+f": (
+        "",
+        "dialog_fill",
+        "control Fill",
+    ),
+    "ctrl+i": ("element* select^",),
+    "ctrl+d": ("element copy",),
+    "ctrl+g": ("planz clear copy preprocess validate blob preopt optimize spool0",),
+    "ctrl+o": (
+        "",
+        "outline",
+    ),
+    "ctrl+r": ("rect 0 0 1000 1000",),
+    "ctrl+s": (
+        "",
+        "dialog_stroke",
+        "control Stroke",
+    ),
+    "ctrl+v": ("clipboard paste",),
+    "ctrl+x": ("clipboard cut",),
+    "ctrl+z": ("reset",),
+    "ctrl+1": ("bind 1 move $x $y",),
+    "ctrl+2": ("bind 2 move $x $y",),
+    "ctrl+3": ("bind 3 move $x $y",),
+    "ctrl+4": ("bind 4 move $x $y",),
+    "ctrl+5": ("bind 5 move $x $y",),
+    "ctrl+f3": (
+        "",
+        "rotaryview",
+    ),
+    "ctrl+f9": (
+        "",
+        "dialog_flip",
+        "control Flip",
+    ),
+    "ctrl+alt+d": ("image wizard Gold",),
+    "ctrl+alt+e": ("image wizard Simple",),
+    "ctrl+alt+g": (
+        "",
+        "image wizard Gold",
+    ),
+    "ctrl+alt+n": ("image wizard Newsy",),
+    "ctrl+alt+o": (
+        "image wizard Stipo",
+        "outline -1mm",
+    ),
+    "ctrl+alt+s": (
+        "",
+        "image wizard Stipo",
+    ),
+    "ctrl+alt+x": ("image wizard Xin",),
+    "ctrl+alt+y": ("image wizard Gravy",),
+    "ctrl+shift+h": ("scale -1 1",),
+    "ctrl+shift+o": ("outline 1mm",),
+    "ctrl+shift+v": ("scale 1 -1",),
+    "ctrl+alt+shift+escape": (
+        "",
+        "reset_bind_alias",
+    ),
+    "ctrl+alt+shift+home": ("bind default;alias default",),
 }
 DEFAULT_ALIAS = {
     "+scale_up": ("loop scale 1.02",),
@@ -94,16 +212,21 @@ DEFAULT_ALIAS = {
     "-left": ("end left 1mm",),
     "-up": ("end up 1mm",),
     "-down": ("end down 1mm",),
-    "terminal_ruida": ("", "window open Terminal;ruidaserver",),
+    "terminal_ruida": (
+        "",
+        "window open Terminal;ruidaserver",
+    ),
     "terminal_watch": (
         "",
         "window open Terminal;channel save usb;channel save send;channel save recv",
     ),
-    "reset_bind_alias": ("bind default;alias default",),
+    "reset_bind_alias": (
+        "",
+        "bind default;alias default",
+    ),
 }
 
 
-
 def plugin(kernel, lifecycle=None):
     if lifecycle == "register":
         kernel.register("modifier/BindAlias", BindAlias)
@@ -136,12 +259,23 @@ class BindAlias(Modifier):
             context = self.context
             _ = self.context._
             if len(args) == 0:
-                channel(_("----------"))
                 channel(_("Binds:"))
-                for i, key in enumerate(context.keymap):
+
+                def keymap_index(key):
+                    mods, key = key.rsplit("+", 1) if "+" in key else ("", key)
+                    return (
+                        mods,
+                        len(key) if len(key) <= 2 else 3,
+                        key,
+                    )
+
+                channel(_("    Key                    Command"))
+                for i, key in enumerate(
+                    sorted(context.keymap.keys(), key=keymap_index)
+                ):
                     value = context.keymap[key]
-                    channel(_("%d: key %s %s") % (i, key.ljust(15), value))
-                channel(_("----------"))
+                    channel("%2d: %s %s" % (i, key.ljust(22), value))
+                channel("----------")
             else:
                 key = args[0].lower()
                 if key == "default":
@@ -184,12 +318,24 @@ class BindAlias(Modifier):
             context = self.context
             _ = self.context._
             if alias is None:
-                channel(_("----------"))
-                channel(_("Aliases:"))
-                for i, key in enumerate(context.alias):
+                reverse_keymap = {v: k for k, v in context.keymap.items()}
+                channel(_("Aliases (keybind)`:"))
+                channel(_("    Alias                  Command(s)"))
+                for i, key in enumerate(
+                    sorted(
+                        context.alias.keys(),
+                        key=lambda x: x if x[0] not in "+-" else x[1:] + x[0],
+                    )
+                ):
                     value = context.alias[key]
-                    channel("%d: %s %s" % (i, key.ljust(15), value))
-                channel(_("----------"))
+                    keystroke = reverse_keymap[key] if key in reverse_keymap else ""
+                    if keystroke and len(key) + len(keystroke) < 18:
+                        key += " (%s)" % keystroke
+                        keystroke = ""
+                    channel("%2d: %s %s" % (i, key.ljust(22), value))
+                    if keystroke:
+                        channel("    (%s)" % keystroke)
+                channel("----------")
                 return
             alias = alias.lower()
             if alias == "default":
@@ -248,15 +394,20 @@ class BindAlias(Modifier):
         context = self.context
         context.keymap.clear()
         prefs = context.derive("keymap")
-        prefs.kernel.load_persistent_string_dict(prefs.path, context.keymap, suffix=True)
+        prefs.kernel.load_persistent_string_dict(
+            prefs.path, context.keymap, suffix=True
+        )
         if not len(context.keymap):
             self.default_keymap()
             return
+        # Remap "control+" to "ctrl+"
+        for key in list(context.keymap.keys()):
+            if key.startswith("control+"):
+                newkey = "ctrl+" + key[8:]
+                context.keymap[newkey] = context.keymap[key]
+                del context.keymap[key]
         for key, values in DEFAULT_KEYMAP.items():
-            if (
-                not key in context.keymap
-                or context.keymap[key] in values[1:]
-            ):
+            if not key in context.keymap or context.keymap[key] in values[1:]:
                 value = values[0]
                 if value:
                     context.keymap[key] = value
@@ -272,10 +423,7 @@ class BindAlias(Modifier):
             self.default_alias()
             return
         for key, values in DEFAULT_ALIAS.items():
-            if (
-                not key in context.alias
-                or context.alias[key] in values[1:]
-            ):
+            if not key in context.alias or context.alias[key] in values[1:]:
                 value = values[0]
                 if value:
                     context.alias[key] = value
@@ -295,3 +443,20 @@ class BindAlias(Modifier):
             value = values[0]
             if value:
                 self.context.alias[key] = value
+
+
+def keymap_execute(context, keyvalue, keydown=True):
+    """
+    Execute keybind accelerator if it exists and return true
+
+    Else return false
+    """
+    if keyvalue not in context.keymap:
+        return False
+    action = context.keymap[keyvalue]
+    if keydown or action.startswith("+"):
+        if not keydown and action.startswith("+"):
+            action = "-" + action[1:]
+        for cmd in action.split(";"):
+            context("%s\n" % cmd)
+    return True
