@@ -1,5 +1,5 @@
 from abc import ABC
-from typing import Any, Callable, Dict, Generator, Optional, Tuple, Union
+from typing import Optional
 
 from ..device.lasercommandconstants import (
     COMMAND_CUT,
@@ -13,7 +13,16 @@ from ..device.lasercommandconstants import (
     COMMAND_SET_INCREMENTAL,
 )
 from ..svgelements import Color, Path, Point
-from ..tools.rasterplotter import X_AXIS, BOTTOM, Y_AXIS, RIGHT, LEFT, UNIDIRECTIONAL, TOP, RasterPlotter
+from ..tools.rasterplotter import (
+    BOTTOM,
+    LEFT,
+    RIGHT,
+    TOP,
+    UNIDIRECTIONAL,
+    X_AXIS,
+    Y_AXIS,
+    RasterPlotter,
+)
 from ..tools.zinglplotter import ZinglPlotter
 
 """
@@ -45,6 +54,8 @@ class LaserSettings:
 
         self.force_twitchless = False
         self.raster_alt = False
+        self.constant_move_x = False
+        self.constant_move_y = False
         self.raster_step = 0
         self.raster_direction = 1  # Bottom To Top - Default.
         self.raster_swing = False  # False = bidirectional, True = Unidirectional
@@ -102,11 +113,15 @@ class LaserSettings:
 
     @property
     def horizontal_raster(self):
-        return self.raster_step and (self.raster_direction == 0 or self.raster_direction == 1)
+        return self.raster_step and (
+            self.raster_direction == 0 or self.raster_direction == 1
+        )
 
     @property
     def vertical_raster(self):
-        return self.raster_step and (self.raster_direction == 2 or self.raster_direction == 3)
+        return self.raster_step and (
+            self.raster_direction == 2 or self.raster_direction == 3
+        )
 
     @property
     def implicit_accel(self):
@@ -253,7 +268,7 @@ class CutObject:
                 if c.burn_started:
                     return True
             elif c.burns_done == c.passes:
-                    return True
+                return True
         return False
 
     def contains_unburned_groups(self):
@@ -282,8 +297,13 @@ class CutGroup(list, CutObject, ABC):
     """
 
     def __init__(
-        self, parent, children=(),
-        settings=None, passes=1, constrained=False, closed=False,
+        self,
+        parent,
+        children=(),
+        settings=None,
+        passes=1,
+        constrained=False,
+        closed=False,
     ):
         list.__init__(self, children)
         CutObject.__init__(self, parent=parent, settings=settings, passes=passes)
@@ -329,7 +349,11 @@ class CutGroup(list, CutObject, ABC):
             for s in c.flat():
                 yield s
 
-    def candidate(self, complete_path: Optional[bool]=False, grouped_inner: Optional[bool]=False):
+    def candidate(
+        self,
+        complete_path: Optional[bool] = False,
+        grouped_inner: Optional[bool] = False,
+    ):
         """
         Candidates are CutObjects:
         1. That do not contain one or more unburned inner constrained cutcode objects.
@@ -378,11 +402,7 @@ class CutGroup(list, CutObject, ABC):
             # if this is not a closed path we should only yield first and last segments
             # Planner will need to determine which end of the subpath is yielded
             # and only consider the direction starting from the end
-            if (
-                complete_path
-                and not grp.closed
-                and isinstance(grp, CutGroup)
-            ):
+            if complete_path and not grp.closed and isinstance(grp, CutGroup):
                 if grp[0].burns_done < grp[0].passes:
                     yield grp[0]
                 # Do not yield same segment a 2nd time if only one segment
@@ -587,18 +607,44 @@ class CutCode(CutGroup):
 
 class LineCut(CutObject):
     def __init__(self, start_point, end_point, settings=None, passes=1, parent=None):
-        CutObject.__init__(self, start_point, end_point, settings=settings, passes=passes, parent=parent)
+        CutObject.__init__(
+            self,
+            start_point,
+            end_point,
+            settings=settings,
+            passes=passes,
+            parent=parent,
+        )
         settings.raster_step = 0
 
+    def __repr__(self):
+        return f'LineCut({repr(self.start())}, {repr(self.end())}, settings="{self.settings}", passes={self.passes})'
+
     def generator(self):
+        # pylint: disable=unsubscriptable-object
         start = self.start()
         end = self.end()
         return ZinglPlotter.plot_line(start[0], start[1], end[0], end[1])
 
 
 class QuadCut(CutObject):
-    def __init__(self, start_point, control_point, end_point, settings=None, passes=1, parent=None):
-        CutObject.__init__(self, start_point, end_point, settings=settings, passes=passes, parent=parent)
+    def __init__(
+        self,
+        start_point,
+        control_point,
+        end_point,
+        settings=None,
+        passes=1,
+        parent=None,
+    ):
+        CutObject.__init__(
+            self,
+            start_point,
+            end_point,
+            settings=settings,
+            passes=passes,
+            parent=parent,
+        )
         settings.raster_step = 0
         self._control = control_point
 
@@ -611,6 +657,7 @@ class QuadCut(CutObject):
         )
 
     def generator(self):
+        # pylint: disable=unsubscriptable-object
         start = self.start()
         c = self.c()
         end = self.end()
@@ -625,8 +672,24 @@ class QuadCut(CutObject):
 
 
 class CubicCut(CutObject):
-    def __init__(self, start_point, control1, control2, end_point, settings=None, passes=1, parent=None):
-        CutObject.__init__(self, start_point, end_point, settings=settings, passes=passes, parent=parent)
+    def __init__(
+        self,
+        start_point,
+        control1,
+        control2,
+        end_point,
+        settings=None,
+        passes=1,
+        parent=None,
+    ):
+        CutObject.__init__(
+            self,
+            start_point,
+            end_point,
+            settings=settings,
+            passes=passes,
+            parent=parent,
+        )
         settings.raster_step = 0
         self._control1 = control1
         self._control2 = control2
@@ -667,7 +730,9 @@ class RasterCut(CutObject):
     this is a crosshatched cut or not.
     """
 
-    def __init__(self, image, tx, ty, settings=None, crosshatch=False, passes=1, parent=None):
+    def __init__(
+        self, image, tx, ty, settings=None, crosshatch=False, passes=1, parent=None
+    ):
         CutObject.__init__(self, settings=settings, passes=passes, parent=parent)
         assert image.mode in ("L", "1")
         self.first = True  # Raster cuts are always first within themselves.
@@ -843,23 +908,41 @@ class PlotCut(CutObject):
     def __len__(self):
         return len(self.plot)
 
+    def __str__(self):
+        parts = list()
+        parts.append("{points} points".format(points=len(self.plot)))
+        parts.append("xmin: {v}".format(v=self.min_x))
+        parts.append("ymin: {v}".format(v=self.min_y))
+        parts.append("xmax: {v}".format(v=self.max_x))
+        parts.append("ymax: {v}".format(v=self.max_y))
+        return "PlotCut(%s)" % ", ".join(parts)
+
     def check_if_rasterable(self):
         """
-        Rasterable plotcuts must have a max step of less than 15 and must have an unused travel direction.
+        Rasterable plotcuts are heuristically defined as having a max step of less than 15 and
+        must have an unused travel direction.
 
         @return: whether the plot can travel
         """
+        # Default to vector settings.
         self.settings.raster_alt = False
+        self.settings.constant_move_x = False
+        self.settings.constant_move_y = False
         self.settings.raster_step = 0
-        self.settings.force_twitchless = True
-        if not self.travels_left and not self.travels_right and not self.travels_bottom and not self.travels_top:
+        if self.settings.speed < 80:
+            # Twitchless gets sketchy at 80.
+            self.settings.force_twitchless = True
             return False
+            # if self.max_dy >= 15 and self.max_dy >= 15:
+            #     return False  # This is probably a vector.
+        # Above 80 we're likely dealing with a raster.
         if 0 < self.max_dx <= 15:
             self.vertical_raster = True
-        elif 0 < self.max_dy <= 15:
+            self.settings.constant_move_y = True
+        if 0 < self.max_dy <= 15:
             self.horizontal_raster = True
-        else:
-            return False
+            self.settings.constant_move_x = True
+        # if self.vertical_raster or self.horizontal_raster:
         self.settings.raster_step = min(self.max_dx, self.max_dy)
         self.settings.raster_alt = True
         return True
@@ -897,10 +980,15 @@ class PlotCut(CutObject):
             self.max_y = y
 
     def major_axis(self):
-        if self.horizontal_raster:
-            return 0
+        """
+        If both vertical and horizontal are set we prefer vertical as major axis because vertical rastering is heavier
+        with the movement of the gantry bar.
+        @return:
+        """
         if self.vertical_raster:
             return 1
+        if self.horizontal_raster:
+            return 0
 
         if len(self.plot) < 2:
             return 0
@@ -959,7 +1047,7 @@ class PlotCut(CutObject):
         last_y = None
         for x, y, on in self.plot:
             if last_x is not None:
-                length += Point.distance((x,y), (last_x, last_y))
+                length += Point.distance((x, y), (last_x, last_y))
             last_x = 0
             last_y = 0
         return length
@@ -984,59 +1072,15 @@ class PlotCut(CutObject):
         last_yy = None
         ix = 0
         iy = 0
-        # last_dx = None
-        # last_dy = None
         for x, y, on in self.plot:
             idx = int(round(x - ix))
             idy = int(round(y - iy))
             ix += idx
             iy += idy
             if last_xx is not None:
-                # if self.horizontal_raster and idx:
-                #     if idx > 0 > last_dx or idx < 0 < last_dx:
-                #         # If this idx is different direction as the last one, we step y first
-                #         if idy:
-                #             # step y
-                #             for zx, zy in ZinglPlotter.plot_line(last_xx, last_yy, last_xx, iy):
-                #                 yield zx, zy, on
-                #         # step x
-                #         for zx, zy in ZinglPlotter.plot_line(last_xx, iy, ix, iy):
-                #             yield zx, zy, on
-                #     else:
-                #         # If this idx is the same direction as the last one, we step x first
-                #         # step x
-                #         for zx, zy in ZinglPlotter.plot_line(last_xx, last_yy, ix, last_yy):
-                #             yield zx, zy, on
-                #         if idy:
-                #             # step y
-                #             for zx, zy in ZinglPlotter.plot_line(ix, last_yy, ix, iy):
-                #                 yield zx, zy, on
-                # elif self.vertical_raster and idy:
-                #     if idy > 0 > last_dy or idy < 0 < last_dy:
-                #         # If this idy is different direction as the last one, we step x first
-                #         if idx:
-                #             # step x
-                #             for zx, zy in ZinglPlotter.plot_line(last_xx, last_yy, ix, last_yy):
-                #                 yield zx, zy, on
-                #         # step y
-                #         for zx, zy in ZinglPlotter.plot_line(ix, last_yy, ix, iy):
-                #             yield zx, zy, on
-                #     else:
-                #         # If this idy is the same direction as the last one, we step y first
-                #         # step y
-                #         for zx, zy in ZinglPlotter.plot_line(last_xx, last_yy, last_xx, iy):
-                #             yield zx, zy, on
-                #         if idx:
-                #             #step y
-                #             for zx, zy in ZinglPlotter.plot_line(ix, last_yy, ix, iy):
-                #                 yield zx, zy, on
-                # else:
-                #     # Non-raster go directly to result.
                 for zx, zy in ZinglPlotter.plot_line(last_xx, last_yy, ix, iy):
                     yield zx, zy, on
             last_xx = ix
             last_yy = iy
-            # last_dx = idx
-            # last_dy = idy
 
         return self.plot
