@@ -3,7 +3,7 @@ import math
 import wx
 
 from ..core.elements import LaserOperation
-from ..svgelements import Length, Group
+from ..svgelements import Group, Length
 from .icons import icons8_laser_beam_52
 from .mwindow import MWindow
 from .propertiespanel import PropertiesPanel
@@ -144,6 +144,8 @@ class PlannerPanel(wx.Panel):
             dlg.Destroy()
             return
         dlg.Destroy()
+        if cols == 0:
+            return
 
         dlg = wx.TextEntryDialog(self, _("How many copies high?"), _("Enter Rows"), "")
         if dlg.ShowModal() == wx.ID_OK:
@@ -156,6 +158,8 @@ class PlannerPanel(wx.Panel):
             dlg.Destroy()
             return
         dlg.Destroy()
+        if rows == 0:
+            return
 
         elems = []
         cutplan = self.context.default_plan()
@@ -187,7 +191,7 @@ class PlannerPanel(wx.Panel):
             if height:
                 height = "%.1f%s" % (height / conversion, name)
             if width:
-                width  = "%.1f%s" % (width / conversion, name)
+                width = "%.1f%s" % (width / conversion, name)
         dlg.SetValue(str(width) if width is not None else "%f%%" % (100.0 / cols))
         bed_dim = self.context.root
         bed_dim.setting(int, "bed_width", 310)
@@ -355,6 +359,7 @@ class PlannerPanel(wx.Panel):
         self.context.setting(bool, "opt_reduce_directions", False)
         self.context.setting(bool, "opt_remove_overlap", False)
         self.context.setting(bool, "opt_rapid_between", True)
+        self.context.setting(bool, "opt_rasters_split", True)
         self.context.setting(int, "opt_jog_minimum", 256)
         self.context.setting(int, "opt_jog_mode", 0)
 
@@ -458,9 +463,20 @@ class ExecuteJob(MWindow):
         self.panel = PlannerPanel(
             self, wx.ID_ANY, context=self.context, plan_name=plan_name
         )
-        self.panel.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel)
-        self.panel.list_operations.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel.list_operations)
-        self.panel.list_command.Bind(wx.EVT_RIGHT_DOWN, self.on_menu, self.panel.list_command)
+        self.panel.Bind(wx.EVT_LEFT_DOWN, self.on_mouse_left_down, self.panel)
+        self.panel.Bind(wx.EVT_RIGHT_DOWN, self.on_menu_request, self.panel)
+        self.panel.list_operations.Bind(
+            wx.EVT_LEFT_DOWN, self.on_mouse_left_down, self.panel.list_operations
+        )
+        self.panel.list_operations.Bind(
+            wx.EVT_RIGHT_DOWN, self.on_menu_request, self.panel.list_operations
+        )
+        self.panel.list_command.Bind(
+            wx.EVT_LEFT_DOWN, self.on_mouse_left_down, self.panel.list_command
+        )
+        self.panel.list_command.Bind(
+            wx.EVT_RIGHT_DOWN, self.on_menu_request, self.panel.list_command
+        )
         _icon = wx.NullIcon
         _icon.CopyFromBitmap(icons8_laser_beam_52.GetBitmap())
         self.SetIcon(_icon)
@@ -479,12 +495,17 @@ class ExecuteJob(MWindow):
         # MENUBAR END
         # ==========
 
-    def on_menu(self, event):
+    def on_mouse_left_down(self, event):
+        # Convert mac Control+left click into right click
+        if event.RawControlDown() and not event.ControlDown():
+            self.on_menu_request(event)
+        else:
+            event.Skip()
+
+    def on_menu_request(self, event):
         from .wxutils import create_menu_for_choices
 
-        menu = create_menu_for_choices(
-            self, self.context.registered["choices/planner"]
-        )
+        menu = create_menu_for_choices(self, self.context.registered["choices/planner"])
         self.PopupMenu(menu)
         menu.Destroy()
 
