@@ -28,8 +28,8 @@ class LhystudiosConfigurationPanel(wx.Panel):
         self.checkbox_fix_speeds = wx.CheckBox(
             self, wx.ID_ANY, _("Fix rated to actual speed")
         )
-        self.checkbox_twitchless = wx.CheckBox(
-            self, wx.ID_ANY, _("Reduce vector twitching (BETA)")
+        self.checkbox_twitchfull = wx.CheckBox(
+            self, wx.ID_ANY, _("Revert to legacy vector twitching")
         )
         self.checkbox_alternative_raster = wx.CheckBox(
             self, wx.ID_ANY, _("Alternative Rastering (BETA)")
@@ -39,7 +39,9 @@ class LhystudiosConfigurationPanel(wx.Panel):
         self.checkbox_flip_y = wx.CheckBox(self, wx.ID_ANY, _("Flip Y"))
         self.checkbox_home_bottom = wx.CheckBox(self, wx.ID_ANY, _("Home Bottom"))
         self.checkbox_swap_xy = wx.CheckBox(self, wx.ID_ANY, _("Swap X and Y"))
-        self.checkbox_strict = wx.CheckBox(self, wx.ID_ANY, _("Strict Programmed Speed Mode"))
+        self.checkbox_strict = wx.CheckBox(
+            self, wx.ID_ANY, _("Strict Programmed Speed Mode")
+        )
         self.spin_home_x = wx.SpinCtrl(
             self, wx.ID_ANY, "0.0", min=-50000.0, max=50000.0
         )
@@ -71,8 +73,10 @@ class LhystudiosConfigurationPanel(wx.Panel):
         self.Bind(wx.EVT_CHECKBOX, self.on_check_home_bottom, self.checkbox_home_bottom)
         self.Bind(wx.EVT_CHECKBOX, self.on_check_swapxy, self.checkbox_swap_xy)
         self.Bind(wx.EVT_CHECKBOX, self.on_check_strict, self.checkbox_strict)
-        self.Bind(wx.EVT_CHECKBOX, self.on_check_twitchless, self.checkbox_twitchless)
-        self.Bind(wx.EVT_CHECKBOX, self.on_check_alt_raster, self.checkbox_alternative_raster)
+        self.Bind(wx.EVT_CHECKBOX, self.on_check_twitchfull, self.checkbox_twitchfull)
+        self.Bind(
+            wx.EVT_CHECKBOX, self.on_check_alt_raster, self.checkbox_alternative_raster
+        )
         self.Bind(wx.EVT_SPINCTRL, self.spin_on_home_x, self.spin_home_x)
         self.Bind(wx.EVT_TEXT, self.spin_on_home_x, self.spin_home_x)
         self.Bind(wx.EVT_SPINCTRL, self.spin_on_home_y, self.spin_home_y)
@@ -154,18 +158,13 @@ class LhystudiosConfigurationPanel(wx.Panel):
                 + "if you could leave a comment on Github issue #{issue}."
             ).format(issue=773)
         )
-        self.checkbox_twitchless.SetToolTip(
+        self.checkbox_twitchfull.SetToolTip(
             _(
                 "Twitching is an unnecessary move in an unneeded direction at the start and end of travel moves between vector burns. "
-                + "It is most noticeable when you are doing a number of small burns (e.g. stitch holes in leather). "
-                + "This option changes how travel moves are performed and result in a noticeable faster travel time. "
+                "It is most noticeable when you are doing a number of small burns (e.g. stitch holes in leather). "
+                "A revised twitchless mode is now default in 0.7.6 or later which results in a noticeable faster travel time. "
+                "This option allows you to revert to the previous mode if you experience problems."
             )
-            + "\n\n"
-            + _(
-                "If this works OK for you please leave it on. "
-                + "If you have any problems, we would very much appreciate it "
-                + "if you could leave a comment on Github issue #{issue}."
-            ).format(issue=772)
         )
         self.spin_home_x.SetMinSize((80, 23))
         self.spin_home_x.SetToolTip(_("Translate Home X"))
@@ -184,7 +183,7 @@ class LhystudiosConfigurationPanel(wx.Panel):
                     _(
                         "It works by swapping adjacent on or off bits to group on and off together and reduce the number of switches "
                         + "e.g. instead of 1010 it will burn 1100. "
-                        + "Because the laser beam is overlapping, and because a bit is only moved at most 1/1000\", "
+                        + 'Because the laser beam is overlapping, and because a bit is only moved at most 1/1000", '
                         + "the difference should not be visible even under magnification."
                     ),
                     _(
@@ -213,10 +212,12 @@ class LhystudiosConfigurationPanel(wx.Panel):
             _("Current number of bytes in the write buffer.")
         )
         self.spin_packet_buffer_max.SetToolTip(_("Current maximum write buffer limit."))
-        self.checkbox_autolock.SetToolTip(_(
-            "Leave the steppers energised and the rail locked after the burn has finished. "
-            + "By default, if this option is not selected, the rail is unlocked at the end of the burn."
-        ))
+        self.checkbox_autolock.SetToolTip(
+            _(
+                "Leave the steppers energised and the rail locked after the burn has finished. "
+                + "By default, if this option is not selected, the rail is unlocked at the end of the burn."
+            )
+        )
         self.checkbox_autolock.SetValue(1)
         # end wxGlade
 
@@ -256,8 +257,8 @@ class LhystudiosConfigurationPanel(wx.Panel):
         sizer_board_1.Add(self.checkbox_fix_speeds, 0, *checkbox_border)
         sizer_board_1.Add(self.checkbox_strict, 0, *checkbox_border)
 
+        sizer_board_2.Add(self.checkbox_twitchfull, 0, *checkbox_border)
         sizer_board_2.Add(self.checkbox_alternative_raster, 0, *checkbox_border)
-        sizer_board_2.Add(self.checkbox_twitchless, 0, *checkbox_border)
 
         sizer_board.Add(sizer_board_1, 1, 0, 0)
         sizer_board.Add(sizer_board_2, 1, wx.EXPAND | wx.LEFT, 10)
@@ -324,7 +325,7 @@ class LhystudiosConfigurationPanel(wx.Panel):
         context.setting(bool, "home_bottom", False)
         context.setting(bool, "strict", False)
         context.setting(bool, "nse_raster", False)
-        context.setting(bool, "twitchless", False)
+        context.setting(bool, "twitchfull", False)
 
         context.setting(int, "home_adjust_x", 0)
         context.setting(int, "home_adjust_y", 0)
@@ -344,7 +345,7 @@ class LhystudiosConfigurationPanel(wx.Panel):
         self.checkbox_home_right.SetValue(context.home_right)
         self.checkbox_home_bottom.SetValue(context.home_bottom)
         self.checkbox_strict.SetValue(context.strict)
-        self.checkbox_twitchless.SetValue(context.twitchless)
+        self.checkbox_twitchfull.SetValue(context.twitchfull)
         self.checkbox_alternative_raster.SetValue(context.nse_raster)
 
         self.spin_home_x.SetValue(context.home_adjust_x)
@@ -379,8 +380,8 @@ class LhystudiosConfigurationPanel(wx.Panel):
     def on_check_strict(self, event=None):
         self.context.strict = self.checkbox_strict.GetValue()
 
-    def on_check_twitchless(self, event=None):
-        self.context.twitchless = self.checkbox_twitchless.GetValue()
+    def on_check_twitchfull(self, event=None):
+        self.context.twitchfull = self.checkbox_twitchfull.GetValue()
 
     def on_check_alt_raster(self, event=None):
         self.context.nse_raster = self.checkbox_alternative_raster.GetValue()
