@@ -1,5 +1,6 @@
 import threading
 import time
+from math import isinf, isnan
 
 import wx
 
@@ -9,6 +10,7 @@ from meerk40t.gui.laserrender import (
     DRAW_MODE_INVERT,
     DRAW_MODE_REFRESH,
 )
+from meerk40t.gui.wxutils import get_key_name
 from meerk40t.gui.zmatrix import ZMatrix
 from meerk40t.kernel import Job, Module
 from meerk40t.svgelements import Matrix, Point, Viewbox
@@ -37,7 +39,6 @@ ORIENTATION_GRID = 0b00000100000000
 ORIENTATION_NO_BUFFER = 0b00001000000000
 BUFFER = 10.0
 
-
 # TODO: _buffer can be updated partially rather than fully rewritten, especially with some layering.
 
 
@@ -96,6 +97,9 @@ class ScenePanel(wx.Panel):
         self.scene_panel.Bind(wx.EVT_PAINT, self.on_paint)
         self.scene_panel.Bind(wx.EVT_ERASE_BACKGROUND, self.on_erase)
 
+        self.scene_panel.Bind(wx.EVT_KEY_DOWN, self.on_key_down)
+        self.scene_panel.Bind(wx.EVT_KEY_UP, self.on_key_up)
+
         self.scene_panel.Bind(wx.EVT_MOTION, self.on_mouse_move)
 
         self.scene_panel.Bind(wx.EVT_MOUSEWHEEL, self.on_mousewheel)
@@ -141,6 +145,56 @@ class ScenePanel(wx.Panel):
         """
         self.scene._signal_widget(self.scene.widget_root, *args, **kwargs)
 
+    # Indicator for Keyboard-Modifier
+    isShiftPressed = False
+    isCtrlPressed = False
+    isAltPressed = False
+
+    def on_key_down(self, evt):
+        literal = get_key_name(evt, True)
+        keycode = evt.GetKeyCode()
+        # print("Key-Down: %f - literal: %s" % (keycode, literal))
+        if "shift+" in literal:
+            if not self.isShiftPressed:  # ignore multiple calls
+                self.isShiftPressed = True
+                self.scene.event(self.scene.last_position, "kb_shift_press")
+        if "ctrl+" in literal:
+            if not self.isCtrlPressed:  # ignore multiple calls
+                self.isCtrlPressed = True
+                self.scene.event(self.scene.last_position, "kb_ctrl_press")
+        if "alt+" in literal:
+            if not self.isAltPressed:  # ignore multiple calls
+                self.isAltPressed = True
+                self.scene.event(self.scene.last_position, "kb_alt_press")
+                # self.scene.event(self.scene.last_position, "kb_alt_press")
+
+    def on_key_up(self, evt):
+        # literal = get_key_name(evt, True) # Until the behaviour of multi-keypresses has been clarified
+        literal = ""
+        keystates = (evt.ShiftDown(), evt.ControlDown(), evt.AltDown())
+        if self.isShiftPressed and not keystates[0]:
+            literal += "shift+"
+        if self.isCtrlPressed and not keystates[1]:
+            literal += "ctrl+"
+        if self.isAltPressed and not keystates[2]:
+            literal += "alt+"
+
+        key = evt.GetKeyCode()
+        # print("Key-Up: %f - literal: %s" % (key, literal))
+
+        if "shift+" in literal:
+            if self.isShiftPressed:  # ignore multiple calls
+                self.isShiftPressed = False
+                self.scene.event(self.scene.last_position, "kb_shift_release")
+        if "ctrl+" in literal:
+            if self.isCtrlPressed:  # ignore multiple calls
+                self.isCtrlPressed = False
+                self.scene.event(self.scene.last_position, "kb_ctrl_release")
+        if "alt+" in literal:
+            if self.isAltPressed:  # ignore multiple calls
+                self.isAltPressed = False
+                self.scene.event(self.scene.last_position, "kb_alt_release")
+
     def on_size(self, event=None):
         if self.context is None:
             return
@@ -218,7 +272,12 @@ class ScenePanel(wx.Panel):
         """
         Scene Panel left click event for down.
         """
+        # Convert mac Control+left click into right click
+        if event.RawControlDown() and not event.ControlDown():
+            self.on_right_mouse_down(event)
+            return
         self.SetFocus()
+
         if not self.scene_panel.HasCapture():
             self.scene_panel.CaptureMouse()
         self.scene.event(event.GetPosition(), "leftdown")
@@ -227,6 +286,10 @@ class ScenePanel(wx.Panel):
         """
         Scene Panel left click event for up.
         """
+        # Convert mac Control+left click into right click
+        if event.RawControlDown() and not event.ControlDown():
+            self.on_right_mouse_up(event)
+            return
         if self.scene_panel.HasCapture():
             self.scene_panel.ReleaseMouse()
         self.scene.event(event.GetPosition(), "leftup")
@@ -239,6 +302,8 @@ class ScenePanel(wx.Panel):
             return
         self.scene.event(event.GetPosition(), "doubleclick")
 
+    last_mode = None
+
     def on_mouse_move(self, event: wx.MouseEvent):
         """
         Scene Panel move event. Calls hover if the mouse has no pressed buttons.
@@ -675,6 +740,7 @@ class Scene(Module, Job):
         """
         if self.log_events:
             self.log_events("%s: %s" % (event_type, str(window_pos)))
+
         if window_pos is None:
             # Capture Lost
             for i, hit in enumerate(self.hit_chain):
@@ -700,6 +766,39 @@ class Scene(Module, Job):
             previous_top_element = self.hit_chain[0][0]
         except (IndexError, TypeError):
             previous_top_element = None
+
+        if event_type in (
+            "kb_shift_release",
+            "kb_shift_press",
+            "kb_ctrl_release",
+            "kb_ctrl_press",
+            "kb_alt_release",
+            "kb_alt_press",
+        ):
+            # print("Keyboard-Event raised: %s" % event_type)
+            self.rebuild_hittable_chain()
+            for current_widget, current_matrix in self.hittable_elements:
+                space_pos = window_pos
+                if current_matrix is not None and not current_matrix.is_identity():
+                    space_cur = current_matrix.point_in_inverse_space(window_pos[0:2])
+                    space_last = current_matrix.point_in_inverse_space(window_pos[2:4])
+                    sdx = space_cur[0] - space_last[0]
+                    sdy = space_cur[1] - space_last[1]
+                    space_pos = (
+                        space_cur[0],
+                        space_cur[1],
+                        space_last[0],
+                        space_last[1],
+                        sdx,
+                        sdy,
+                    )
+                try:
+                    # We ignore the consume etc. for the time being...
+                    response = current_widget.event(window_pos, space_pos, event_type)
+                except AttributeError:
+                    pass
+            return
+
         if event_type in (
             "leftdown",
             "middledown",
@@ -711,6 +810,7 @@ class Scene(Module, Job):
             self.time = time.time()
             self.rebuild_hittable_chain()
             self.find_hit_chain(window_pos)
+
         for i, hit in enumerate(self.hit_chain):
             if hit is None:
                 continue  # Element was dropped.
@@ -754,10 +854,20 @@ class Scene(Module, Job):
                     self.log_events("Converted %s: %s" % ("leftclick", str(window_pos)))
             else:
                 response = current_widget.event(window_pos, space_pos, event_type)
+
+            # if current_widget is None:
+            #    widgetname = "undefined"
+            # else:
+            #    widgetname = type(current_widget).__name__
+
             if response == RESPONSE_ABORT:
+                # f event_type == "leftdown":
+                #    print("Event was aborted by %s" % widgetname)
                 self.hit_chain.clear()
                 return
             elif response == RESPONSE_CONSUME:
+                # if event_type == "leftdown":
+                #    print("Event was consumed by %s" % widgetname)
                 return
             elif response == RESPONSE_CHAIN:
                 continue
@@ -774,20 +884,26 @@ class Scene(Module, Job):
         """
         import platform
 
+        custom1 = -1
+        custom2 = -2
         if cursor == "sizing":
             new_cursor = wx.CURSOR_SIZING
         elif cursor in ("size_nw", "size_se"):
             new_cursor = wx.CURSOR_SIZENWSE
         elif cursor in ("size_sw", "size_ne"):
             new_cursor = wx.CURSOR_SIZENESW
-        elif cursor in ("size_n", "size_s"):
+        elif cursor in ("size_n", "size_s", "size_ns"):
             new_cursor = wx.CURSOR_SIZENS
-        elif cursor in ("size_e", "size_w"):
+        elif cursor in ("size_e", "size_w", "size_ew"):
             new_cursor = wx.CURSOR_SIZEWE
         elif cursor == "arrow":
             new_cursor = wx.CURSOR_ARROW
         elif cursor == "cross":
-            new_cursor = wx.CROSS_CURSOR
+            new_cursor = wx.CURSOR_CROSS
+        elif cursor == "rotate1":
+            new_cursor = wx.CURSOR_CROSS
+        elif cursor == "rotate2":
+            new_cursor = wx.CURSOR_CROSS
         else:
             new_cursor = wx.CURSOR_ARROW
             self.log("Invalid cursor.")
@@ -971,13 +1087,7 @@ class Widget(list):
         """
         if dx == 0 and dy == 0:
             return
-        if dx == float("nan"):
-            return
-        if dy == float("nan"):
-            return
-        if abs(dx) == float("inf"):
-            return
-        if abs(dy) == float("inf"):
+        if isnan(dx) or isnan(dy) or isinf(dx) or isinf(dy):
             return
         self.translate_loop(dx, dy)
 
@@ -1328,6 +1438,7 @@ class SceneSpaceWidget(Widget):
 
         If nothing was otherwise hit by the event, we process the scene manipulation events
         """
+
         if event_type == "hover":
             return RESPONSE_CHAIN
         if self.aspect:
@@ -1435,7 +1546,7 @@ class SceneSpaceWidget(Widget):
             )
             p /= 250.0
             if self._previous_zoom is not None:
-                zoom_factor = e ** p
+                zoom_factor = e**p
                 zoom_change = zoom_factor / self._previous_zoom
                 self._previous_zoom = zoom_factor
                 self.scene_widget.matrix.post_scale(
