@@ -1,8 +1,9 @@
 import wx
 from wx import aui
 
+from ..core.bindalias import keymap_execute
 from ..core.cutcode import CutCode
-from ..core.elements import LaserOperation, isDot, ConsoleOperation
+from ..core.elements import ConsoleOperation, LaserOperation, isDot
 from ..svgelements import (
     SVG_ATTR_STROKE,
     Color,
@@ -24,7 +25,7 @@ from .icons import (
     icons8_system_task_20,
     icons8_vector_20,
 )
-from .laserrender import DRAW_MODE_ICONS, DRAW_MODE_TREE, swizzlecolor
+from .laserrender import DRAW_MODE_ICONS, swizzlecolor
 from .mwindow import MWindow
 from .wxutils import create_menu, get_key_name
 
@@ -106,24 +107,15 @@ class TreePanel(wx.Panel):
 
     def on_key_down(self, event):
         keyvalue = get_key_name(event)
-        keymap = self.context.keymap
-        if keyvalue in keymap:
-            action = keymap[keyvalue]
-            self.context(action + "\n")
-        else:
-            event.Skip()
+        if keymap_execute(self.context, keyvalue, keydown=True):
+            return
+        event.Skip()
 
     def on_key_up(self, event):
         keyvalue = get_key_name(event)
-        keymap = self.context.keymap
-        if keyvalue in keymap:
-            action = keymap[keyvalue]
-            if action.startswith("+"):
-                # Keyup commands only trigger if the down command started with +
-                action = "-" + action[1:]
-                self.context(action + "\n")
-        else:
-            event.Skip()
+        if keymap_execute(self.context, keyvalue, keydown=False):
+            return
+        event.Skip()
 
     def initialize(self):
         self.context.listen("rebuild_tree", self.on_rebuild_tree_signal)
@@ -192,11 +184,7 @@ class TreePanel(wx.Panel):
         :param args:
         :return:
         """
-        if self.context.draw_mode & DRAW_MODE_TREE != 0:
-            self.wxtree.Hide()
-            return
-        else:
-            self.wxtree.Show()
+        # self.wxtree.Show()
         self.shadow_tree.rebuild_tree()
         self.request_refresh()
 
@@ -574,6 +562,8 @@ class ShadowTree:
         @return:
         """
         item = node.item
+        if item is None:
+            raise ValueError("Item was None for node " + repr(node))
         if not item.IsOk():
             raise ValueError("Bad Item")
         node.unregister_object()
@@ -737,14 +727,18 @@ class ShadowTree:
         self.wxtree.SetItemText(node.item, node.label)
         try:
             stroke = node.object.stroke
-            color = wx.Colour(swizzlecolor(Color(stroke).argb))
-            self.wxtree.SetItemTextColour(node.item, color)
+            wxcolor = Color(stroke).bgr
+            if wxcolor is not None:
+                color = wx.Colour(wxcolor)
+                self.wxtree.SetItemTextColour(node.item, color)
         except AttributeError:
             pass
         try:
             color = node.color
-            c = wx.Colour(swizzlecolor(Color(color)))
-            self.wxtree.SetItemTextColour(node.item, c)
+            wxcolor = Color(color).bgr
+            if wxcolor is not None:
+                c = wx.Colour(wxcolor)
+                self.wxtree.SetItemTextColour(node.item, c)
         except AttributeError:
             pass
 
