@@ -2,7 +2,7 @@ import os
 from copy import copy
 from os import path as ospath
 
-from ..core.planner import make_actual, needs_actualization
+from ..core.cutplan import make_actual, needs_actualization
 from ..svgelements import Angle, Color, Length, Matrix, Path, SVGImage
 from .actualize import actualize
 
@@ -37,6 +37,9 @@ def plugin(kernel, lifecycle=None):
     def image(command, channel, _, data_type=None, data=None, args=tuple(), **kwargs):
         if data_type == "inkscape":
             inkscape_path, filename = data
+            if filename is None:
+                channel(_("File was not set."))
+                return
             if filename.endswith("png"):
                 from PIL import Image
 
@@ -888,7 +891,7 @@ def plugin(kernel, lifecycle=None):
                 element_bottom.image_width,
                 element_bottom.image_height,
             ) = element_bottom.image.size
-            element_bottom.transform.pre_translate(0,y)
+            element_bottom.transform.pre_translate(0, y)
 
             if hasattr(element, "node"):
                 element.node.remove_node()
@@ -1020,15 +1023,16 @@ def plugin(kernel, lifecycle=None):
             w, h = im.size
             from PIL import Image
 
-            def t(i):
-                return int(i * w / (points - 1))
+            def t(i, width):
+                return int(i * width / (points - 1))
 
-            def x(i):
-                return int(w * float(args[i + 1]))
+            def x(i, width):
+                return int(width * float(args[i + 1]))
 
-            boxes = list((t(i), 0, t(i + 1), h) for i in range(points - 1))
+            boxes = list((t(i, w), 0, t(i + 1, w), h) for i in range(points - 1))
             quads = list(
-                (x(i), 0, x(i), h, x(i + 1), h, x(i + 1), 0) for i in range(points - 1)
+                (x(i, w), 0, x(i, w), h, x(i + 1, w), h, x(i + 1, w), 0)
+                for i in range(points - 1)
             )
             mesh = list(zip(boxes, quads))
             element.image = im.transform(im.size, Image.MESH, mesh, Image.BILINEAR)
@@ -1471,6 +1475,8 @@ class RasterScripts:
         else:
             half_tone = Image.new("L", size)
         draw = ImageDraw.Draw(half_tone)
+        if sample == 0:
+            sample = 1
         for x in range(0, image.size[0], sample):
             for y in range(0, image.size[1], sample):
                 box = image.crop(
