@@ -1,6 +1,6 @@
 import socket
 
-from .kernel import STATE_TERMINATE, Module
+from .kernel import STATE_END, STATE_TERMINATE, Module
 
 
 def plugin(kernel, lifecycle=None):
@@ -91,7 +91,7 @@ class UDPServer(Module):
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         self.socket.settimeout(2)
         self.socket.bind(("", self.port))
-        self.context.threaded(self.run_udp_listener, thread_name=name)
+        self.context.threaded(self.run_udp_listener, thread_name=name, daemon=True)
 
     def finalize(self, *args, **kwargs):
         _ = self.context._
@@ -102,6 +102,7 @@ class UDPServer(Module):
         if self.socket is not None:
             self.socket.close()
             self.socket = None
+        self.state = STATE_TERMINATE
 
     def send(self, message):
         _ = self.context._
@@ -118,12 +119,10 @@ class UDPServer(Module):
         _ = self.context._
         try:
             self.events_channel(_("UDP Socket(%d) Listening.") % self.port)
-            while True:
+            while self.state != STATE_END and self.state != STATE_TERMINATE:
                 try:
                     message, address = self.socket.recvfrom(1024)
                 except (socket.timeout, AttributeError):
-                    if self.state == STATE_TERMINATE:
-                        return
                     continue
                 if address is not None:
                     self.udp_address = address
