@@ -8,6 +8,7 @@ for full details.
 
 import argparse
 import asyncio
+import ctypes
 import os
 import os.path
 import platform
@@ -17,7 +18,7 @@ from .core.exceptions import Mk40tImportAbort
 from .kernel import Kernel
 
 APPLICATION_NAME = "MeerK40t"
-APPLICATION_VERSION = "0.7.5000"
+APPLICATION_VERSION = "0.7.6000"
 
 if not getattr(sys, "frozen", False):
     # If .git directory does not exist we are running from a package like pypi
@@ -99,6 +100,13 @@ parser.add_argument(
     action="store_true",
     help="Do not load meerk40t.plugins entrypoints",
 )
+parser.add_argument(
+    "-A",
+    "--disable-ansi",
+    action="store_true",
+    default=False,
+    help="Disable ANSI colors",
+)
 
 
 def run():
@@ -125,7 +133,13 @@ def run():
         path = "profile%d" % args.profile
     else:
         path = ""
-    kernel = Kernel(APPLICATION_NAME, APPLICATION_VERSION, APPLICATION_NAME, path)
+    kernel = Kernel(
+        APPLICATION_NAME,
+        APPLICATION_VERSION,
+        APPLICATION_NAME,
+        path,
+        ansi=not args.disable_ansi,
+    )
 
     """
     These are frozen bootstraps. They are not dynamically found by entry points they are the configured accepted
@@ -282,7 +296,9 @@ def run():
                 pass
             except pkg_resources.VersionConflict as e:
                 print(
-                    "Cannot install plugin - '{entrypoint}' due to version conflict.".format(entrypoint=str(entry_point))
+                    "Cannot install plugin - '{entrypoint}' due to version conflict.".format(
+                        entrypoint=str(entry_point)
+                    )
                 )
                 print(e)
             else:
